/*****************************************************************************/
/* Kernel Hooks Interface.                                    		     */
/* Version 1.4                                                               */
/* Author: Richard J Moore richardj_moore@uk.ibm.com                         */
/* 	   Vamsi Krishna S. r1vamsi@in.ibm.com 				     */
/*                                                                           */
/* A sample kernel module with hooks in it (in the testhook function).   */
/*                                                                           */
/* Refer to Documentation/hook/HOWTO for details.                            */
/*****************************************************************************/
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/stddef.h>
#include <linux/unistd.h>
#include <linux/hook.h>

#define TEST_HOOK1 test_hook1
#define TEST_HOOK2 test_hook2
#define TEST_HOOK3 test_hook3
#define TEST_HOOK4 test_hook4
#define TEST_HOOK5 test_hook5
DECLARE_HOOK_HEAD(TEST_HOOK1);
DECLARE_HOOK_HEAD(TEST_HOOK2);
DECLARE_HOOK_HEAD(TEST_HOOK3);
DECLARE_HOOK_HEAD(TEST_HOOK4);
DECLARE_HOOK_HEAD(TEST_HOOK5);
DECLARE_EXCLUSIVE_HOOK_HEAD(EX_TEST_HOOK3);
DECLARE_EXCLUSIVE_HOOK_HEAD(EX_TEST_HOOK4);
DECLARE_EXCLUSIVE_HOOK_HEAD(EX_TEST_HOOK5);

int init_module(void)
{
	int rc;

	rc = hook_initialise(TEST_HOOK1);
	DPRINTK("hook_initialise returned %u\n", rc);
	if(rc) 
		return 1;

	rc = hook_initialise(TEST_HOOK2);
	DPRINTK("hook_initialise returned %u\n", rc);
	if(rc) {
		hook_terminate(TEST_HOOK1, 0);
		return 1;
	}
	hook_initialise(TEST_HOOK3);
	hook_initialise(TEST_HOOK4);
	hook_initialise(TEST_HOOK5);
	hook_initialise(EX_TEST_HOOK3);
	hook_initialise(EX_TEST_HOOK4);
	hook_initialise(EX_TEST_HOOK5);

	DPRINTK("now load kexit.o to test the hook(s)\n");
	return 0;
}

void cleanup_module(void)
{
	int rc;

	rc = hook_terminate(TEST_HOOK1, 0);
	DPRINTK("hook_terminate returned %u\n", rc);

	rc = hook_terminate(TEST_HOOK2, 0);
	DPRINTK("hook_terminate returned %u\n", rc);

	hook_terminate(TEST_HOOK3, 0);
	hook_terminate(TEST_HOOK4, 0);
	hook_terminate(TEST_HOOK5, 0);
	hook_terminate(EX_TEST_HOOK3, 0);
	hook_terminate(EX_TEST_HOOK4, 0);
	hook_terminate(EX_TEST_HOOK5, 0);

	return;
}

/*
 * this is an example of how the hooks would be coded in the kernel or a
 * kernel module.
 */
int testhook(void)
{
	printk("testhook entered\n");
	
	HOOK_RET(TEST_HOOK1);
	HOOK(TEST_HOOK2);

	HOOK(TEST_HOOK3);
	HOOK(TEST_HOOK4, 4);
	HOOK(TEST_HOOK5, 5, 0);

	EXCLUSIVE_HOOK(EX_TEST_HOOK3);
	EXCLUSIVE_HOOK(EX_TEST_HOOK4, 4);
	EXCLUSIVE_HOOK(EX_TEST_HOOK5, "%s, %d", "test_hook3", 1);

	printk("testhook exited\n");
	return 0;
}

EXPORT_SYMBOL_NOVERS(testhook);
